# methods to handle dependencies

depend_get_current_build()
{
    printf "%s" "${DEPEND}"
}

depend_get_single_build()
{
    single=$1
    (
    load_env ${single}
    printf "%s" "${DEPEND}"
    ) || nonfail
}

depend_get_single_runtime()
{
    single=$1
    (
    load_env ${single}
    printf "%s" "${RDEPEND}"
    ) || nonfail
}

depend_get_single_host()
{
    single=$1
    (
    load_env ${single}
    printf "%s" "${HDEPEND}"
    ) || nonfail
}



depend_detect_compare_method()
{
    depend=$1

    if test "${depend}" != "${depend#'>='}" ; then
        printf "%s" "depend_compare_ge"
    elif test "${depend}" != "${depend#'>'}" ; then
        printf "%s" "depend_compare_gt"
    elif test "${depend}" != "${depend#'<='}" ; then
        printf "%s" "depend_compare_le"
    elif test "${depend}" != "${depend#'<'}" ; then
        printf "%s" "depend_compare_lt"
    elif test "${depend}" != "${depend#'='}" ; then
        printf "%s" "depend_compare_e"
    else
        printf "%s" "depend_compare_ge"
    fi
}

depend_detect_version_boundary()
{
    depend="$1"

    if test "${depend}" != "${depend#'>='}" ; then
        printf "%s" '>='
    elif test "${depend}" != "${depend#'>'}" ; then
        printf "%s" '>'
    elif test "${depend}" != "${depend#'<='}" ; then
        printf "%s" '<='
    elif test "${depend}" != "${depend#'<'}" ; then
        printf "%s" '<'
    elif test "${depend}" != "${depend#'='}" ; then
        printf "%s" '='
    fi
}

depend_get_pack_noboundary()
{
    depend="$1"

    if test "${depend}" != "${depend#'>='}" ; then
        printf "%s" "${depend#'>='}"
    elif test "${depend}" != "${depend#'>'}" ; then
        printf "%s" "${depend#'>'}"
    elif test "${depend}" != "${depend#'<='}" ; then
        printf "%s" "${depend#'<='}"
    elif test "${depend}" != "${depend#'<'}" ; then
        printf "%s" "${depend#'<'}"
    elif test "${depend}" != "${depend#'='}" ; then
        printf "%s" "${depend#'='}"
    else
        printf "%s" "${depend}"
    fi
}

depend_get_pack_version()
{
    pack=$1
    version=${pack##$(depend_get_pack_name $pack)-}
    if test "${version}" = "${version#[0-9]}" ; then
        version='0.0'
    fi
    printf "%s" ${version}
}

depend_get_pack_name()
{
    pack=$1
    pack=${pack%%-[0-9][^-]*}
    pack=${pack%%-[0-9]}

    printf "%s" "${pack}"
}

depend_compare_ge()
{
    current=$1
    base=$2

    current=${current#'>='}
    current_version=$(depend_get_pack_version ${current})
    current_name=$(depend_get_pack_name ${current})

    available_packs_with_name=${T}/depend_packs_with_name
    : > ${available_packs_with_name}

    ( grep -e "^${current_name}-[0-9]" -e "^${current_name}\$" ${base} ; echo ${current} ) \
        | compare_by_version > ${available_packs_with_name}
    candidate="$(tail -1 ${available_packs_with_name})"
    if test "${candidate}" != "${current}" ; then
        printf "%s" ${candidate}
        return
    else
        if test "$(tail -2 ${available_packs_with_name})" = "$(tail -1 ${available_packs_with_name})" ; then
            return
        fi
        check_position="$(tail -2 ${available_packs_with_name} | head -1)"
        if test "${check_position}" = "${candidate}" ; then
            printf "%s" ${candidate}
            return
        fi
    fi
}

depend_compare_gt()
{
    current=$1
    base=$2

    current=${current#'>'}
    current_version=$(depend_get_pack_version ${current})
    current_name=$(depend_get_pack_name ${current})

    available_packs_with_name=${T}/depend_packs_with_name
    : > ${available_packs_with_name}

    ( grep -e "^${current_name}-[0-9]" -e "^${current_name}\$" ${base} ; echo ${current} ) \
        | compare_by_version > ${available_packs_with_name}
    candidate="$(tail -1 ${available_packs_with_name})"
    if test "${candidate}" != "${current}" ; then
        printf "%s" ${candidate}
    fi
}

depend_compare_e()
{
    current=$1
    base=$2

    current=${current#'='}
    candidate="$(grep -e "^${current}\$" ${base})"
    if test -n "${candidate}" ; then
        printf "%s" ${candidate}
    fi
}

depend_compare_le()
{
    current=$1
    base=$2

    current=${current#'<='}
    current_version=$(depend_get_pack_version ${current})
    current_name=$(depend_get_pack_name ${current})

    available_packs_with_name=${T}/depend_packs_with_name
    : > ${available_packs_with_name}

    ( grep -e "^${current_name}-[0-9]" -e "^${current_name}\$" ${base} ; echo ${current} ) \
        | compare_by_version > ${available_packs_with_name}

    candidate_block="$( grep -n -e "${current}" ${available_packs_with_name} )"
    linebreak="
"
    check_multiline="${candidate_block%%${linebreak}*}"
    if test "${check_multiline}" != "${candidate_block}" ; then
        printf "%s" "${current}"
    else
        numline=${candidate_block%:*}
        if test ${numline} -le 1 ; then
            return
        fi
        numline=$((numline - 1))
        candidate="$(sed -n -e"${numline}p" ${available_packs_with_name} )"
        printf "%s" "${candidate}"
    fi
}

depend_compare_lt()
{
    current=$1
    base=$2

    current=${current#'<'}
    current_version=$(depend_get_pack_version ${current})
    current_name=$(depend_get_pack_name ${current})

    available_packs_with_name=${T}/depend_packs_with_name
    : > ${available_packs_with_name}

    ( grep -e "^${current_name}-[0-9]" -e "^${current_name}\$" ${base} ; echo ${current} ) \
        | compare_by_version > ${available_packs_with_name}

    candidate_block="$( grep -n -e "${current}" ${available_packs_with_name} )"
    linebreak="
"
    candidate_block="${candidate_block%%${linebreak}*}"
    numline=${candidate_block%:*}
    if test ${numline} -le 1 ; then
        return
    fi
    numline=$((numline - 1))
    candidate="$(sed -n -e"${numline}p" ${available_packs_with_name} )"
    printf "%s" "${candidate}"
}

depend_list_available()
{
    mkdir -p ${T}
    available_current="${T}/available_current"
    : > ${available_current}
    if test -d ${IMGDIR} ; then
        find ${IMGDIR} -maxdepth 1 -type d   | while read line ; do
            basename ${line} >> ${available_current}
        done
    fi

    printf "%s" "${available_current}"
}

depend_list_available_desired()
{
    mkdir -p ${T}
    build_available="${T}/build_available"
    : > ${build_available}
    all_profs="$(printf "%s" "${AUSE}" | tr ' ' ',' )"

    entry_all="${any_list_all}"
    # if any_list_all is not defined, leave entry_all blank on purpose.
    # in that case lsrdd will output the default listing,
    # which often is the needed thing
    lsrdd ${all_profs} ${entry_all} >> ${build_available}

    printf "%s" "${build_available}"
}

depend_resolve_current_atom()
{
    depends_current="$( depend_get_current_build )"
    available_current="$( depend_list_available )"
    nonresolved=
    resolved=

    for single_depend in ${depends_current} ; do
        method="$(depend_detect_compare_method ${single_depend})"
        resolved_single_depend="$( ${method} ${single_depend} ${available_current} )"

        if test -z "${resolved_single_depend}" ; then
            nonresolved="${nonresolved} ${single_depend}"
        else
            resolved="${resolved} ${resolved_single_depend}"
        fi
    done

    if test ! -z "${nonresolved}" ; then
        printf "%s\n" "--nonresolved"

        message="\
Build dependencies below were not resolved. You should make needed packages available.
${nonresolved}
"
        awarn "${message}"
        return 1
    fi

    printf "%s" "${resolved}"
}

depend_list_host()
{
    mkdir -p ${T}
    host_current="${T}/host_current"
    : > "${host_current}"
    ls /var/pkgdb/ >> "${host_current}"

    printf "%s" "${host_current}"
}

depend_get_required_host()
{
    host_total=
    local_required=
    if [ -n "${any_list_required}" ] ; then
        local_required="${any_list_required}"
    else
        local_required="${rdd_list_entry}"
    fi
    for host_pack in $(lsrdd ${rdd_prf_entry} ${local_required}) ; do
        host_total="${host_total}
$(depend_get_single_host $host_pack)"
    done
    printf "%s\n" ${host_total} | sort | uniq
}

depend_resolve_host()
{
    method_to_list="depend_list_host"
    depends_current="$( depend_get_required_host )"
    available_current="$( ${method_to_list} )"
    nonresolved=

    for single_depend in ${depends_current} ; do
        method="$(depend_detect_compare_method ${single_depend})"
        resolved_single_depend="$( ${method} ${single_depend} ${available_current} )"

        if test -z "${resolved_single_depend}" ; then
            single_pack="$(depend_get_pack_noboundary ${single_depend})"
            single_pack="$(depend_get_pack_name ${single_pack})"
            nonresolved="${nonresolved}
${single_pack}"
        fi
    done

    printf "%s\n" "${nonresolved}"
}

depend_get_current_runtime_binary()
{
    packs_built_with="$(root_get_installed | tr ' ' '\n')"

    cand_rundeps="${RDEPEND}"
    if test -z "${cand_rundeps}" ; then
        cand_rundeps="${DEPEND}"
    fi
    for rdepend in ${cand_rundeps} ; do

        skip_to_next=
        modificator=$(depend_detect_version_boundary ${rdepend})
        record_from_rdepend=${rdepend#${modificator}}
        name_from_rdepend=$(depend_get_pack_name ${record_from_rdepend})
        version_from_rdepend=$(depend_get_pack_version ${record_from_rdepend})
        pack_buildtime=$(printf "%s\n" "${packs_built_with}" | \
            grep -e "^${name_from_rdepend}-[0-9]" | tail -1)

        pack_rundep=
        if test -n "${pack_buildtime}" ; then
            version_buildtime="$(depend_get_pack_version ${pack_buildtime})"
            record_build=
            for record_build in $(cat "${ROOT}/pkgdb/${pack_buildtime}") ; do
                name_runtime="${record_build%:*}"
                version_runtime="${record_build#*:}"
                if test "${name_runtime}" = "${version_runtime}" ; then
                    version_runtime=
                fi

                if test "${name_runtime}" = 'NONE' ; then
                    skip_to_next=1
                    break
                fi
                if test -z "${name_runtime}" ; then
                    name_runtime="${name_from_rdepend}"
                fi
                if test -z "${version_runtime}" ; then
                    version_runtime="${version_buildtime}"
                fi
                if test -z "${modificator}" ; then
                    modificator='>='
                fi
                case ${modificator} in
                    '>='|'>')
                        pack_rundep=">=${name_runtime}-${version_runtime}"
                        ;;
                    '<='|'<')
                        pack_rundep=">=${name_runtime}-${version_runtime}
                            ${modificator}${name_runtime}-${version_from_rdepend}"
                        ;;
                    *)
                        pack_rundep="${modificator}${name_runtime}-${version_from_rdepend}"
                        ;;
                esac

                if test -n "${pack_rundep}" ; then
                    skip_to_next=1
                    printf "%s\n" "${pack_rundep}"
                fi
            done

            if test -n "${skip_to_next}" ; then
                continue
            fi
            if test -z "${modificator}" ; then
                modificator='>='
            fi
            case ${modificator} in
                '>='|'>')
                    pack_rundep=">=${pack_buildtime}"
                    ;;
                '<='|'<')
                    pack_rundep=">=${pack_buildtime}
                        ${rdepend}"
                    ;;
                *)
                    pack_rundep="${rdepend}"
                    ;;
            esac
        fi
        if test -z "${pack_rundep}" ; then
            pack_rundep="${rdepend}"
        fi

        printf "%s\n" "${pack_rundep}"

    done
}
