# for creating binary package

pack_name()
{
    if [ "${BP}" != "${P}" ] ; then
        for entry in ${BP} ; do
            printf "%s\n" "${entry}"
        done
    else
        for entry in ${BPN} ; do
            printf "%s\n" "${entry}-${BPV}"
        done
    fi
}

pack_name_base()
{
    if [ "${BP}" != "${P}" ] ; then
        for entry in ${BP} ; do
            printf "%s\n" "$(depend_get_pack_name ${entry})"
        done
    else
        for entry in ${BPN} ; do
            printf "%s\n" "${entry}"
        done
    fi
}

pack_name_file()
{
    flavour_string=
    for fl_word in ${FLAVOUR} ; do
        flavour_string="${flavour_string}${fl_word}-"
    done
    postfix="${flavour_string}${ANYARCH}"
    if [ -n "${ANYOS}" ] ; then
        postfix="${postfix}-${ANYOS}"
    fi
    inner_version="$(fetch_inner_version)"
    if [ -n "${inner_version}" ] ; then
        postfix="${postfix}-${inner_version}"
    fi
    postfix="${postfix}.tgz"

    if [ "${BP}" != "${P}" ] ; then
        for entry in ${BP} ; do
            printf "%s\n" "${entry}-${postfix}"
        done
    else
        for entry in ${BPN} ; do
            printf "%s\n" "${entry}-${BPV}-${postfix}"
        done
    fi
}

pack_name_file_actual()
{
    keywords_check
    status_bool || return
    pack_name_file
}

pack_fallback_allowed()
{
    # if current package produces single binary archive, fallback materials can be used for it:
    # ${D} instead of ${IMGDID}/${packname} for binaries,
    # postinst script name instead of ${packname}.postinst and so on

    fallback_allowed=
    if test "${BP}" != ${P} ; then
        if test "${BP#* }" = "${BP}" ; then
            fallback_allowed=1
        fi
    elif test "${BPN#* }" = "${BPN}" ; then
        fallback_allowed=1
    fi

    if test -n "${fallback_allowed}" ; then
        printf "%s\n" "${fallback_allowed}"
    fi
}

pack_dir_for_runtime_entry()
{
    pack="$1"
    fallback="$2"

    dir=
    if test -d ${IMGDIR}/${pack} ; then
        dir="${IMGDIR}/${pack}"
    elif test -n "${fallback}" -a -d "${D}" ; then
        dir="${D}"
    fi
    if test -n "${dir}" ; then
        printf "%s\n" "${dir}"
    fi
}

pack_metafile_for_runtime_entry()
{
    pack="$1"
    script="$2"
    fallback="$3"

    filename=
    if test -f "${DISTDIR}/${pack}.${script}" ; then
        filename="${DISTDIR}/${pack}.${script}"
    elif test -n "${fallback}" -a -f "${DISTDIR}/${script}" ; then
        filename="${DISTDIR}/${script}"
    fi
    if test -n "${filename}" ; then
        printf "%s\n" "${filename}"
    fi
}

pack_gen_meta()
{
    archives="$(pack_name)"
    fallback="$(pack_fallback_allowed)"

    for pack in ${archives} ; do
        dir="$(pack_dir_for_runtime_entry ${pack} ${fallback})"
        if test -z "${dir}" ; then
            continue
        fi

        (
        cd ${dir}

        mkdir -p var/lib/pkg/files/
        find * > var/lib/pkg/files/${pack}

        basepack="$(depend_get_pack_name ${pack})"
        postinst_file="$(pack_metafile_for_runtime_entry ${basepack} 'postinst' ${fallback} )"
        if test -n "${postinst_file}" ; then
            mkdir -p var/lib/pkg/scripts/
            cp -f ${postinst_file} var/lib/pkg/scripts/${pack}
            chmod 555 var/lib/pkg/scripts/${pack}
        fi
        triggers_file="$(pack_metafile_for_runtime_entry ${basepack} 'triggers' ${fallback} )"
        if test -n "${triggers_file}" ; then
            mkdir -p var/lib/pkg/triggers/
            cp -f ${triggers_file} var/lib/pkg/triggers/${pack}
            chmod 555 var/lib/pkg/triggers/${pack}
        fi
        )
    done
}

pack_create()
{
    archives="$(pack_name_file)"
    packs="$(pack_name)"

    fallback="$(pack_fallback_allowed)"

    mkdir -p ${PACKDIR}
    for entry in ${packs} ; do
        fileentry="$(get_first_string "${archives}")"
        archives="$(delete_first_string "${archives}" "${fileentry}" )"
        if test -z "${fileentry}" ; then
            die "No file for bin package ${entry}"
        fi

        installed_dir="$(pack_dir_for_runtime_entry ${entry} ${fallback})"
        if test -z "${installed_dir}" ; then
            die "No directory to create package from ${entry}"
        fi

        pack_create_archive ${installed_dir} ${PACKDIR}/${fileentry}

    done
}

pack_create_archive()
{
    sourcedir="$1"
    archivefile="$2"

    alog "Creating binary package: plain tar archive ${archivefile}"
    cd ${sourcedir}
    ${UTILROOT} tar czvf ${archivefile} *
}

pack_info()
{
    return
}

pack_get_version()
{
    return
}

pack_get_inner_version()
{
    return
}

pack_define_by_filename()
{
    return
}

pack_unpack()
{
    archives="$(pack_name_file)"

    mkdir -p ${D}
    for entry in ${archives} ; do
        tar xf ${PACKDIR}/${entry} -C ${D}
    done
}

pack_dump_content_archive()
{
    archive="$1"
    dumppath="$2"

    archivename="${archive##*/}"
    tar tzf "${archive}" > "${dumppath}/${archivename}.txt"
}

pack_dump_content()
{
    path_to_dump="$1"
    if test -z "${path_to_dump}" ; then
        path_to_dump="${T}/dumpcontent"
    fi
    mkdir -p ${path_to_dump}

    params=

    if test ! -t 0 ; then
        while read line_input_pack <&0 ; do
            for input_pack in ${line_input_pack} ; do
                params=1
                cur_package="${input_pack}"
                if test ! -f "${cur_package}" ; then
                    cur_package=${PACKDIR}/${cur_package}
                fi
                if test ! -f "${cur_package}" ; then
                    awarn "Package ${input_pack} was not found."
                    continue
                fi
                pack_dump_content_archive ${cur_package} ${path_to_dump}
            done
        done
    fi

    if test -z "${params}" ; then
        pack_name_file | while read input_pack ; do
            cur_package="${input_pack}"
            if test ! -f "${cur_package}" ; then
                cur_package="${PACKDIR}/${cur_package}"
            fi
            if test ! -f "${cur_package}" ; then
                awarn "Package ${input_pack} was not found."
                continue
            fi
            pack_dump_content_archive ${cur_package} ${path_to_dump}
        done
    fi
}

pack_dump_info_archive()
{
    archive="$1"
    dumppath="$2"
    info="$3"

    archivename="${archive##*/}"
    #printf "%s\n" "something" > "${dumppath}/${archivename}.${info}.txt"

    return
}

pack_dump_info()
{
    path_to_dump="$1"
    if test -z "${path_to_dump}" ; then
        path_to_dump="${T}/dumpinfo"
    fi
    mkdir -p ${path_to_dump}

    info="$2"

    return
}
