#!/bin/sh

INSTALL=${INSTALL:-"install"}
COPY=${COPY:-"cp -Rf"}
MKDIR=${MKDIR:-"mkdir -p"}
DESTDIR=${DESTDIR:-"${HOME}/rrr"}
PREFIX=${PREFIX:-} 
P=${P:-"any"}
ENGINEDIR="${ENGINEDIR:-${PREFIX}/lib/any/workdir}"
BINDIR="${BINDIR:-${PREFIX}/bin}"
DOCDIR="${DOCDIR:-${PREFIX}/share/doc/${P}}"
MANDIR="${MANDIR:-${PREFIX}/share/man}"
SUBCOMDIR="${PREFIX}/lib/any"

if [ -f conf/config.sh ] ; then
    . conf/config.sh
fi

install_engine()
{
    ${MKDIR} "${DESTDIR}${ENGINEDIR}/"
    ${COPY} ./. "${DESTDIR}${ENGINEDIR}/"
}

install_system()
{
    ${MKDIR} \
        "${DESTDIR}${DOCDIR}/" \
        "${DESTDIR}${MANDIR}/" \
        "${DESTDIR}${BINDIR}/"

    ${INSTALL} -m644 'README-any' 'License-ISC' "${DESTDIR}${DOCDIR}/"
    if test -d doc/html/ ; then
        ${COPY} doc/html/ "${DESTDIR}${DOCDIR}/"
    fi
    if test -d doc/man/ ; then
        ${COPY} \
            doc/man/man1/ \
            doc/man/man3/ \
            doc/man/man7/ \
            "${DESTDIR}${MANDIR}/"
    fi
    ${INSTALL} -m555 \
        'bin/lspkg' \
        'bin/any' \
        'bin/anch' \
        'bin/chrootenv' \
        'bin/tmpdroprdd' \
        'bin/dumpbuild' \
        "${DESTDIR}${BINDIR}/"

    if test -n "${progchroot_flag}" ; then
        sed -e "s/^PROGCHROOT=.*/PROGCHROOT=\"\${PROGCHROOT:-${progchroot_value}}\"/" \
            "${DESTDIR}${BINDIR}/anch" > "${DESTDIR}${BINDIR}/anch.edit"
        mv -f "${DESTDIR}${BINDIR}/anch.edit" "${DESTDIR}${BINDIR}/anch"
    fi
    if test -n "${progsu_flag}" ; then
        sed -e "s/^PROGSU=.*/PROGSU=\"\${PROGSU:-${progsu_value}}\"/" \
            "${DESTDIR}${BINDIR}/anch" > "${DESTDIR}${BINDIR}/anch.edit"
        mv -f "${DESTDIR}${BINDIR}/anch.edit" "${DESTDIR}${BINDIR}/anch"
    fi
}

install_tools()
{
    for tool in toolsrc/* ; do
        if test -d $tool ; then
        (
            cd $tool
            ./install.sh "$@"
        )
        fi
    done
}

install_commands()
{
    mkdir -p ${DESTDIR}${SUBCOMDIR}
    commands=
    for one in libexec/* ; do
        commands="${commands} ${one}"
    done
    ${INSTALL} -m555 $commands ${DESTDIR}${SUBCOMDIR}
}

set -o errexit
set -o xtrace

WORKDIR=
COMMANDS=
progsu_value=
progsu_flag=
progchroot_value=
progchroot_flag=
while test -n "$1" ; do
    arg="$1"
    case $arg in
        --workdir)
            WORKDIR=1
            break
        ;;
        --commands)
            COMMANDS=1
            break
        ;;
        PROGSU=*)
            progsu_value="${arg#PROGSU=}"
            progsu_flag=1
        ;;
        PROGCHROOT=*)
            progchroot_value="${arg#PROGCHROOT=}"
            progchroot_flag=1
        ;;
    esac
    shift
done

if test -n "${COMMANDS}" ; then
    install_commands
    exit
fi
if test -n "${WORKDIR}" ; then
    install_engine
    exit
fi

if test -z "${WORKDIR}" ; then
    install_system
    install_tools "$@"
fi
